package org.apache.bcel.classfile;

package .var.folders._j.vfbv187d0bb8b8w5spzgg8180000gn.T;

import java.io.DataInput;
import java.io.DataOutputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Iterator;
import java.util.stream.Stream;
import org.apache.bcel.classfile.Attribute;
import org.apache.bcel.classfile.ConstantPool;
import org.apache.bcel.classfile.InnerClass;
import org.apache.bcel.classfile.Visitor;

public final class InnerClasses extends Attribute implements Iterable<InnerClass> {
  private static final InnerClass[] EMPTY_INNER_CLASSE_ARRAY = new InnerClass[0];
  
  private InnerClass[] innerClasses;
  
  public InnerClasses(org.apache.bcel.classfile.InnerClasses c) {
    this(c.getNameIndex(), c.getLength(), c.getInnerClasses(), c.getConstantPool());
  }
  
  InnerClasses(int nameIndex, int length, DataInput input, ConstantPool constantPool) throws IOException {
    this(nameIndex, length, (InnerClass[])null, constantPool);
    int classCount = input.readUnsignedShort();
    this.innerClasses = new InnerClass[classCount];
    for (int i = 0; i < classCount; i++)
      this.innerClasses[i] = new InnerClass(input); 
  }
  
  public InnerClasses(int nameIndex, int length, InnerClass[] innerClasses, ConstantPool constantPool) {
    super((byte)6, nameIndex, length, constantPool);
    this.innerClasses = (innerClasses != null) ? innerClasses : EMPTY_INNER_CLASSE_ARRAY;
  }
  
  public void accept(Visitor v) {
    v.visitInnerClasses(this);
  }
  
  public Attribute copy(ConstantPool constantPool) {
    org.apache.bcel.classfile.InnerClasses c = (org.apache.bcel.classfile.InnerClasses)clone();
    c.innerClasses = new InnerClass[this.innerClasses.length];
    Arrays.setAll(c.innerClasses, i -> this.innerClasses[i].copy());
    c.setConstantPool(constantPool);
    return c;
  }
  
  public void dump(DataOutputStream file) throws IOException {
    super.dump(file);
    file.writeShort(this.innerClasses.length);
    for (InnerClass innerClass : this.innerClasses)
      innerClass.dump(file); 
  }
  
  public InnerClass[] getInnerClasses() {
    return this.innerClasses;
  }
  
  public Iterator<InnerClass> iterator() {
    return Stream.<InnerClass>of(this.innerClasses).iterator();
  }
  
  public void setInnerClasses(InnerClass[] innerClasses) {
    this.innerClasses = (innerClasses != null) ? innerClasses : EMPTY_INNER_CLASSE_ARRAY;
  }
  
  public String toString() {
    StringBuilder buf = new StringBuilder();
    buf.append("InnerClasses(");
    buf.append(this.innerClasses.length);
    buf.append("):\n");
    for (InnerClass innerClass : this.innerClasses)
      buf.append(innerClass.toString(getConstantPool())).append("\n"); 
    return buf.substring(0, buf.length() - 1);
  }
}
